package eu.jonahbauer.raytracing.scene;

import eu.jonahbauer.raytracing.math.AABB;
import eu.jonahbauer.raytracing.math.Ray;
import eu.jonahbauer.raytracing.render.texture.Color;
import eu.jonahbauer.raytracing.scene.util.HittableBinaryTree;
import eu.jonahbauer.raytracing.scene.util.HittableCollection;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public final class Scene extends HittableCollection {
    private final @NotNull HittableCollection objects;
    private final @NotNull SkyBox background;

    private final @Nullable List<@NotNull Target> targets;

    public Scene(@NotNull List<? extends @NotNull Hittable> objects) {
        this(objects, null);
    }

    public Scene(@NotNull List<? extends @NotNull Hittable> objects, @Nullable List<? extends @NotNull Target> targets) {
        this(Color.BLACK, objects, targets);
    }

    public Scene(@NotNull SkyBox background, @NotNull List<? extends @NotNull Hittable> objects) {
        this(background, objects, null);
    }

    public Scene(@NotNull SkyBox background, @NotNull List<? extends @NotNull Hittable> objects, @Nullable List<? extends @NotNull Target> targets) {
        var list = new ArrayList<Hittable>(objects.size() + (targets != null ? targets.size() : 0));
        list.addAll(objects);
        if (targets != null) list.addAll(targets);
        this.objects = new HittableBinaryTree(list);
        this.background = Objects.requireNonNull(background);

        this.targets = targets != null ? List.copyOf(targets) : null;
    }

    @Override
    public void hit(@NotNull Ray ray, @NotNull State state) {
        objects.hit(ray, state);
    }

    @Override
    public @NotNull AABB getBoundingBox() {
        return objects.getBoundingBox();
    }

    public @Nullable List<@NotNull Target> getTargets() {
        return targets;
    }

    public @NotNull Color getBackgroundColor(@NotNull Ray ray) {
        return background.getColor(ray);
    }
}
